#!/usr/bin/python

import time,csv,re,sys,gc
import numpy as np
from htsint.database import get_idmapping_file,get_file_sizes

timeStart = time.time()
totalRecords = 0
idmappingFile = get_idmapping_file()
idmappingFid = open(idmappingFile,'rb')
reader = csv.reader(idmappingFid,delimiter="\t")

print("...gathering data this may take a few minutes")
bad = 0
current = None
noTaxa = 0
noNcbi = 0
noTaxaYesNcbi = 0

ac2kbMap = {}
toAdd = {}
totalUniprot = 0#set([])

def queue_entries(toAdd,ac2kbMap):
    ## do something
    pass
    #if ac2kbMap.has_key('P07663'):
    #    print toAdd['PER_DROME']

for record in reader:

    if len(record) != 3:
        continue

    uniprotKbAc,uniprotKbEntry,ncbiId,refseq,ncbiTaxaId = None,None,None,None,None
    uniprotKbAc = record[0]

    if record[1] == 'NCBI_TaxID':
        ncbiTaxaId = record[2]
    elif record[1] == 'GeneID':
        ncbiId = record[2]
    elif record[1] == 'UniProtKB-ID':
        uniprotKbEntry = record[2]
        if not ac2kbMap.has_key(uniprotKbAc):
            ac2kbMap[uniprotKbAc] = uniprotKbEntry
    elif record[1] == 'RefSeq':
        refseq = record[2]
    else:
        continue

    ## skip the XXXX-1 like uniprot ac
    if ac2kbMap.has_key(uniprotKbAc) == False:
        continue

    ## get current key
    uniprotKbEntry = ac2kbMap[uniprotKbAc] 

    ## make new entry if necessary
    if uniprotKbEntry and not toAdd.has_key(uniprotKbEntry):

        ## queue in intervals
        totalRecords += 1 
        if totalRecords % 100000 == 0:
            queue_entries(toAdd,ac2kbMap)
            toAdd,ac2kbMap = {},{}

        #ac2kbMap[uniprotKbAc] = uniprotKbEntry
        #toAdd[uniprotKbEntry] = {'ncbi-taxa-id':None,
        #                         'gene-id':None,
        #                         'uniprot-ac':set([]),
        #                         'refseq':set([])}
        #totalUniprot.update([uniprotKbEntry])

    ## populate uniprot dictionary
    #toAdd[uniprotKbEntry]['uniprot-ac'].update([uniprotKbAc])
    #
    #if ncbiTaxaId:
    #    toAdd[uniprotKbEntry]['ncbi-taxa-id'] = ncbiTaxaId
    #elif ncbiId:
    #    toAdd[uniprotKbEntry]['gene-id'] = ncbiId
    #elif refseq:
    #    toAdd[uniprotKbEntry]['refseq'].update([refseq])

print("run time... %s"%time.strftime('%H:%M:%S',time.gmtime(time.time()-timeStart)))

timeStart = time.time()

#print("......")
print("total: %s"%totalRecords)
#print("noTaxa: %s"%noTaxa)
#print("noNcbi: %s"%noNcbi)
#print("noTaxaYesNcbi: %s"%noTaxaYesNcbi)



#!/usr/bin/env python
"""
Calcuates the pairwise distances between genes 
"""

__author__ = "Adam Richards"

import os,sys,csv,shutil,cPickle,getopt
import numpy as np
import networkx as nx
from basedir import __basedir__
from multiprocessing import Pool, cpu_count, Manager

def mp_worker((source,sink,graphPath)):
    """
    find shortest path length
    """
    G = nx.read_gpickle(graphPath)
    minDistance = 1e8
    if G.has_node(source) and G.has_node(sink):
        (dijkDist, dijkPath) = nx.bidirectional_dijkstra(G,source,sink)
    else:
        dijkDist = None
    
    if dijkDist:
        return [source,sink,dijkDist]

class TermDistances(object):
    """
    a class to represent distances among all gene-ontology terms
    """

    def __init__(self,termsPath,termGraphPath,resultsDir=os.path.join(".","htsint-tmp"),cluster=True):
        """
        Constructor
        
        if cpus > 1 then the script assumes we are in a cluster envrionment
        Note that the results dir will be cleaned before each run
        """

        ## error checking
        for path in [termGraphPath, termsPath]:
            if os.path.exists(path) == False:
                raise Exception("Cannot find specified path\n%s"%path)
        
        ## variables
        self.termsPath = os.path.realpath(termsPath)
        self.termGraphPath = os.path.realpath(termGraphPath)
        self.queue = []
        self.baseDir =  os.path.realpath(os.path.dirname(__file__))
            
        ## results dir will be emptied on each run
        self.resultsDir = os.path.realpath(resultsDir)

        ## load the term graph and the terms
        self.G = nx.read_gpickle(self.termGraphPath)
        tmp = open(self.termsPath,'r')
        self.gene2go,self.go2gene = cPickle.load(tmp)
        tmp.close()

        ## variables
        self.terms = self.go2gene.keys()
        self.totalTerms = float(len(self.terms))
        self.totalDistances = ((self.totalTerms*(self.totalTerms+1))/2) - self.totalTerms

    def create_scripts(self,email,name='dist',cpus=1):
        """
        creates bash scripts to be submitted to the cluster queue
        """

        ## empty the results directory
        if os.path.isdir(self.resultsDir):
            shutil.rmtree(self.resultsDir)
        os.mkdir(self.resultsDir)
        
        ## variables
        chunks = cpus
        script = os.path.join(__basedir__,"TermDistances.py")
        stopPoints = np.arange(0,self.totalDistances,
                               int(round(float(self.totalDistances)/float(chunks))))
        if stopPoints[-1] < self.totalDistances:
            stopPoints = np.hstack([stopPoints[1:],np.array([self.totalDistances])])

        print '... submitting %s jobs'%(len(stopPoints))

        ## create scripts
        begin = 0
        for i,chunk in enumerate(range(stopPoints.size)):
            stop = stopPoints[chunk] 

            submitFile = os.path.join(self.resultsDir,"%s-%s.sh"%(name,i))
            submitLog =  os.path.join(self.resultsDir,"%s-%s.log"%(name,i))
            args = " -f %s -l %s -t '%s' -g '%s' -r %s"%(int(begin),int(stop),self.termsPath,
                                                         self.termGraphPath,self.resultsDir)
            f = open(submitFile, 'w')
            f.write("#!/bin/bash\n" +
                    "#$ -S /bin/bash\n" +
                    "#$ -j yes\n" +
                    "#S -M %s\n"%email +
                    "#$ -o %s\n"%submitLog +
                    "/usr/bin/python "+ script + args)

            f.close()
            self.queue.append(submitFile)
            begin = stop

    def submit(self):
        """
        submit the file to the cluster        
        """
    
        if len(self.queue) == 0:
            raise Exception("There are no script to be submitted to the queue\n did you run create_scripts?")

        for queueFileName in self.queue:
            os.system("qsub " + queueFileName)

    def run(self,first=None,last=None):
        """
        Search for all pairwise shortest paths
        """

        ## create a results file 
        outFile = os.path.join(self.resultsDir,"out-%s-%s.npy"%(first,last))
        mat = np.zeros((last-first,3),).astype(str)
        
        if first == None or last == None:
            first = 0
            last = int(self.totalDistances)

        toRun = range(first,last)

        count = -1
        lineCount = -1
        for i,termI in enumerate(self.terms):
            
            if i % 20 == 0:
                print "%s/%s"%(i,len(self.terms))

            for j,termJ in enumerate(self.terms):
                if j >= i:
                    continue
                count += 1
                if count not in toRun:
                    continue
                
                ## find the distance
                distance = self.get_distance(termI,termJ)

                if distance != None:
                    lineCount+=1
                    mat[lineCount,:] = [termI,termJ,distance]

        np.save(outFile,mat)

    def get_distance(self,source,sink):
        """
        find shortest path length
        """

        minDistance = 1e8
        if self.G.has_node(source) and self.G.has_node(sink):
            (dijkDist, dijkPath) = nx.bidirectional_dijkstra(self.G,source,sink)
            return dijkDist
        return None

    def run_with_multiprocessing(self,resultsFilePath,chunkSize=1000,cpus=7):
        """
        experimental
        """

        ## create a results file )
        outFid = open(resultsFilePath,'w')
        writer = csv.writer(outFid)
        writer.writerow(["i","j","distance"])

        ## assemble the pairwise terms
        print("loading data")
        pairwiseTerms = []
        for i,termI in enumerate(self.terms):
            for j,termJ in enumerate(self.terms):
                if j >= i:
                    continue
                pairwiseTerms.append([termI,termJ,self.termGraphPath])
                
        print("loaded.")

        #####################################
        #print("DEBUG is on")
        #pairwiseTerms = pairwiseTerms[:1000]
        #chunkSize = 300
        ####################################

        start = 0
        stop = chunkSize
        
        numChunks = int(np.ceil(float(len(pairwiseTerms) / float(chunkSize))))
        for chunk in range(numChunks):
            p = Pool(cpus)
            _results = p.map_async(mp_worker,pairwiseTerms[start:start+chunkSize])
            results = _results.get()
            p.close()
            p.join()
 
            for row in results:
                if row:
                    writer.writerow(row)
        
            print("%s"%((start+chunkSize)/float(len(pairwiseTerms)) * 100.0)+"% complete")
            start += chunkSize

        outFid.close()
        #_results = p.map(mp_worker, pairwiseTerms)

if __name__ == "__main__":
    ## read in input file      
    if len(sys.argv) < 3:
        print sys.argv[0] + "-f first -l last -q query_file -d database -e evalue -o outdir"
        sys.exit()

    try:
        optlist, args = getopt.getopt(sys.argv[1:], 'f:l:t:g:r:')
    except getopt.GetoptError:
        raise Exception(sys.argv[0] + "-f first -l last -t termsPath -g termGraphPath -r resultsDir")
        
    first,last,termsPath,termGraphPath,resultsDir = None,None,None,None,None
    for o,a in optlist:
        if o == '-f':
            first = int(a)
        if o == '-l':
            last  = int(a)
        if o == '-t':
            termsPath = a
        if o == '-g':
            termGraphPath  = a
        if o == '-r':
            resultsDir  = a

    td = TermDistances(termsPath,termGraphPath,resultsDir)
    td.run(first=first,last=last)


"""
import multiprocessing

class MyFancyClass(object):
    
    def __init__(self, name):
        self.name = name
    
    def do_something(self):
        proc_name = multiprocessing.current_process().name
        print 'Doing something fancy in %s for %s!' % (proc_name, self.name)


def worker(q):
    obj = q.get()
    obj.do_something()


if __name__ == '__main__':
    queue = multiprocessing.Queue()

    p = multiprocessing.Process(target=worker, args=(queue,))
    p.start()
    
    queue.put(MyFancyClass('Fancy Dan'))
    
    # Wait for the worker to finish
    queue.close()
    queue.join_thread()
    p.join()
"""



import sys,os,cPickle
import numpy as np
import networkx as nx
from sqlalchemy.sql import select
from htsint.database import Base,Taxon,Gene,Uniprot,GoTerm,GoAnnotation,db_connect
from htsint.database import read_ontology_file,fetch_taxa_annotations

"""
Classes used to interact with gene ontology data
The data is stored in the htsint database
"""

class GeneOntology(object):
    "A class to interact with Gene Ontology data"
    
    def __init__(self,taxaList,verbose=False,upass='',idType='ncbi',useIea=True,\
                 aspect='biological_process'):
        """
        Constructor
        
        taxaList a list of NCBI taxa ids
        """

        ## error checking
        idType = idType.lower()
        if idType not in ['uniprot','ncbi']:
            raise Exception("Invalid idType argument in fetch annotations use 'uniprot' or 'ncbi'")

        ## start a database session
        self.session,self.engine = db_connect(verbose=verbose,upass=upass)

        ## global variables
        self.taxaList = taxaList
        self.idType = idType
        self.useIea = useIea
        self.aspect = aspect

    def summarize(self,refTaxon,termsPath):
        """
        GO object summary and sanity check
        """

        refTaxon = str(refTaxon)
        if refTaxon not in self.taxaList:
            raise Exception("refTaxon not present in taxaList")

        conn = self.engine.connect()
        gene2go,go2gene = self.load_dicts(termsPath=termsPath)

        s = select([Taxon.id,Taxon.ncbi_id,Taxon.name]).where(Taxon.ncbi_id.in_(self.taxaList))
        _taxaQueries = conn.execute(s)
        taxaQueries = _taxaQueries.fetchall()
        taxaMap = dict([(str(r['ncbi_id']),str(r['id'])) for r in taxaQueries])

        gene2id = {}
        for tquery in taxaQueries: 
            s = select([Gene.id,Gene.ncbi_id],Gene.taxa_id==tquery['id'])
            _geneQueries = conn.execute(s)
            taxaDict = dict([(str(r['ncbi_id']),str(r['id'])) for r in _geneQueries.fetchall()])
            if str(tquery['ncbi_id']) == refTaxon:
                refGenes = taxaDict.copy()

            print("there are  %s genes from %s (%s)"%(len(taxaDict.keys()),tquery['name'],tquery['ncbi_id']))
            gene2id.update(taxaDict)

        ## check for unmatched genes
        unmatched = 0
        for gene in gene2go.iterkeys():
            if not gene2id.has_key(gene):
                unmatched += 1
        
        print("Summary")
        print("IEA annotations: %s"%self.useIea)
        print("total genes in combined taxa: %s"%(len(gene2id.keys())))
        if unmatched > 0:
            print("WARNING: there were unmatched genes unmatched: %s"%unmatched)
        print("total genes with at least one annotation: %s"%(len(gene2go.keys())))
        print("total unique annotations: %s"%(len(go2gene.keys())))
        print("---------------------")

        _gene2go,_prot2go = fetch_taxa_annotations([refTaxon],self.engine,aspect=self.aspect,\
                                                   useIea=self.useIea)

        total= 0
        for k,v in _gene2go.iteritems():
            total += len(v)
        print('RefTaxa genes: %s'%(len(refGenes.keys())))
        print('Only RefTaxa: %s annotated genes, %s total annotations'%(len(_gene2go.keys()), total))
        
        total= 0
        for k,v in gene2go.iteritems():
            total += len(v)
        print('With additional taxa: %s annotated genes, %s total annotations'%(len(gene2go.keys()), total))
        print('Percent annotation: %s'%(float(len(gene2go.keys())) / float(len(refGenes.keys()))))
                                       

    def check_taxon(self,taxID):
        """
        check if taxon is in database
        """

        taxQuery = self.session.query(Taxon).filter_by(ncbi_id=taxID).first()
        if taxQuery == None:
            raise Exception("Taxon:%s not found in database"%taxID)
        

    def create_dicts(self,termsPath,accepted=None):
        """
        get the go2gene and gene2go dictionaries
        'accepted' - list of genes that restrict included terms to a particular list
        """

        conn = self.engine.connect()

        ## error checking
        if self.aspect not in ['biological_process','molecular_function','cellular_component']:
            raise Exception("Invalid aspect specified%s"%self.aspect)

        ## gene2go
        print("...creating gene2go dictionary -- this may take several minutes or longer on the number of genes")
        _gene2go,prot2go = fetch_taxa_annotations(self.taxaList,self.engine,aspect=self.aspect,\
                                                 useIea=self.useIea)

        print("...creating go2gene dictionary -- this may take several minutes")
        go2gene = {}
        gene2go = {}
        for gene,terms in _gene2go.iteritems():
            if accepted and gene not in accepted:
                continue

            gene2go[gene] = terms
            for term in terms:
                if go2gene.has_key(term) == False:
                    go2gene[term] = set([])
                go2gene[term].update([gene])

        for term,genes in go2gene.iteritems():
            go2gene[term] = list(genes)
        
        ## pickle the dictionaries    
        tmp = open(termsPath,'w')
        cPickle.dump([gene2go,go2gene],tmp)
        tmp.close()

    def load_dicts(self,termsPath=None,log=None):
        """
        get the go2gene and gene2go dictionaries
        """

        if self.aspect not in ['biological_process','molecular_function','cellular_component']:
            raise Exception("Invalid aspect specified%s"%self.aspect)

        if termsPath != None and os.path.exists(termsPath):
            tmp = open(termsPath,'r')
            gene2go,go2gene = cPickle.load(tmp)
            tmp.close()
            return gene2go,go2gene

        return None,None

    def create_gograph(self,termsPath=None,graphPath=None,addShared=False):
        """
        creates the go graph for a given species
        aspect = 'biological_process','molecular_function' or 'cellular_component'

        A go graph can be created with any gene list
        The genes have to be present in the database
        """

        print '...creating gograph'
        ## error checking
        if self.aspect not in ['biological_process','molecular_function','cellular_component']:
            raise Exception("Invalid aspect specified%s"%self.aspect)

        ## load pickled version if present
        if graphPath != None and os.path.exists(graphPath):
            G = nx.read_gpickle(graphPath)
            return G

        ## load all the ontology related information as a set of dictionaries
        gene2go,go2gene = self.load_dicts(termsPath=termsPath)
        if gene2go == None or go2gene == None:
            raise Exception("Could not load gene2go or go2Gene-- do they exist?")

        if len(go2gene.keys()) < 5:
            raise Exception("There are insufficient terms present to create a graph %s"%(len(go2gene.keys())))

        _goDict = read_ontology_file()
        goDict = _goDict[self.aspect]
        
        print "...creating go term graph -- this may take several minutes or hours depending on the number of genes"
        ## calculate the term-term edges using term IC (-ln(p(term)))
        edgeDict = self.get_weights_by_ic(goDict,go2gene)

        if len(edgeDict) == 0:
            raise Exception("No edges were found -- annotation information may be two sparse")

        ## terms without distances set to max
        print("...finding term-term distances")
        allDistances = np.array(edgeDict.values())
        maxDistance = allDistances.max()
        for parent,children in goDict.iteritems():
            for child in list(children):
                if edgeDict.has_key(parent+"#"+child) or edgeDict.has_key(child+"#"+parent):
                    continue
                edgeDict[parent+"#"+child] = maxDistance

        ## add the edges through shared genes
        if addShared == True:
            print('...adding term-term edges through shared genes')
            p5 = np.percentile(allDistances,0.05)
            newEdges = 0
            for termI,genesI in go2gene.iteritems():
                for termJ,genesJ in go2gene.iteritems():

                    if edgeDict.has_key(termI+"#"+termJ) or edgeDict.has_key(termJ+"#"+termI):
                        continue

                    sharedGenes = len(list(set(genesI).intersection(set(genesJ))))
                    if sharedGenes == 0:
                        continue

                    ## add new edge in both directions
                    edgeDict[termI+"#"+termJ] = float(p5) / float(sharedGenes)
                    newEdges += 1

        ## initialize the graph
        G = nx.Graph()
        for nodes,weight in edgeDict.iteritems():
            parent,child = nodes.split("#")
            G.add_edge(parent,child,weight=weight)
        
        def trim_leaf_nodes(G):
            """
            trim nodes with degree = 1 that are not present in go2gene
            """

            degreeDict = nx.degree(G)
            removed = 0
            for node,degree in degreeDict.iteritems():

                if degree > 1:
                    continue

                if go2gene.has_key(node):
                    continue

                removed += 1
                G.remove_node(node)

            return G,removed

        ## trim unecessary leaf nodes
        print("trimming leaf nodes...")
        removed = 1
        print("nodes, edges")
        print("%s, %s"%(G.number_of_nodes(), G.number_of_edges()))
        while removed > 0:
            G,removed = trim_leaf_nodes(G)
            print("%s, %s"%(G.number_of_nodes(), G.number_of_edges()))

        ## trim nodes with degree 2
        print("trimming degree=2 nodes...")
        toRemove = []
        toAdd    = []
        degreeDict = nx.degree(G)
        for node,degree in degreeDict.iteritems():

            if degree != 2:
                continue

            if go2gene.has_key(node):
                continue

            neighbors = [x for x in nx.all_neighbors(G,node)]

            if neighbors[0] in toRemove or neighbors[1] in toRemove:
                continue

            edge1 = G.get_edge_data(neighbors[0],node)['weight']
            edge2 = G.get_edge_data(neighbors[1],node)['weight']

            toRemove.append(node)
            toAdd.append((neighbors[0],neighbors[1],np.array([edge1,edge2]).mean()))

        for node in toRemove:
            G.remove_node(node)
        for edge in toAdd:
            G.add_edge(edge[0],edge[1],weight=edge[2])

        ## trim unecessary leaf nodes
        print("trimming leaf nodes...")
        removed = 1
        print("nodes, edges")
        print("%s, %s"%(G.number_of_nodes(), G.number_of_edges()))
        while removed > 0:
            G,removed = trim_leaf_nodes(G)
            print("%s, %s"%(G.number_of_nodes(), G.number_of_edges()))

        ## save mst to pickle format
        if graphPath != None:
            nx.write_gpickle(G, graphPath)
            print '...saving pickle graph'

        return G

    def get_weights_by_ic(self,goDict,go2gene):
        """
        get the weight of a term using information content
        returns a networkx edge dictionary
        """

        print "...... terms", len(go2gene.keys())
        
        total = 0
        for term,genes in go2gene.iteritems():
            total += len(genes)

        edgeDict = {}
        for parent,children in goDict.iteritems():
            if go2gene.has_key(parent) == False:
                continue

            parentIC = -np.log(float(len(list(go2gene[parent])))  / float(total))
            for child in list(children):
                if go2gene.has_key(child) == False:
                    continue

                childIC = -np.log(float(len(list(go2gene[child])))  / float(total))
                distance = np.abs(parentIC - childIC)
                edgeDict[parent+"#"+child] = distance
        
        return edgeDict

if __name__ == "__main__":
    """
    Xenopus tropicalis      - 8364
    Xenopus laevis          - 8355
    Mus musculus            - 10090
    Drosophila melanogaster - 7227
    """

    ## testing with a species
    go = GeneOntology(['7091'],useIea=True)
    go.create_gograph()




#!/usr/bin/env python
"""
Used to test the database

The schema display is borrowed from:
http://www.sqlalchemy.org/trac/wiki/UsageRecipes/SchemaDisplay
"""

### make imports
import sys,os,re
from DatabaseTables import Base,Taxon,Gene,Uniprot
from DatabaseTools import db_connect

try:
    from sqlalchemy_schemadisplay import create_schema_graph
    createGraph = True
except:
    createGraph = False

## conect to the database                                                            
session,engine = db_connect(verbose=False)

## test the 'Taxon' table
testID = '7227'
query = session.query(Taxon).filter_by(ncbi_id=testID).first() 

if query == None:
    print "ERROR: init taxon not found."
    sys.exit()

if int(query.ncbi_id) != int(testID):
    print "ERROR: Bad match to taxon id"
    print query.ncbi_id,testID

if query.name != "Drosophila melanogaster":
    print "ERROR: Bad match to taxon name"
    print query.name
    sys.exit()

if "fruit fly" not in [query.common_name_1, query.common_name_2, query.common_name_3]:
    print "ERROR: Bad match to common name"
    sys.exit()

## test the 'Gene' table
if session.query(Gene).count() < 4:
    print "ERROR: not enough genes found"
    sys.exit()

query = session.query(Gene).filter_by(ncbi_id='3771877').first() 

if query.symbol != 'Adh':
    print "ERROR: bad gene symbol found"
    print query.symbol
    sys.exit()

## test the 'Uniprot' table
if session.query(Uniprot).count() < 100:
    print "ERROR: not enough uniprot found"
    sys.exit()

if createGraph == True:
    # create the pydot graph object by autoloading all tables via a bound metadata object
    graph = create_schema_graph(metadata=Base.metadata,
                                show_datatypes=False,   # can get large with datatypes
                                show_indexes=False,     # ditto for indexes
                                rankdir='LR',           # From left to right (instead of top to bottom)
                                concentrate=False       # Don't try to join the relation lines together 
                            )
    graph.write_svg('dbschema.svg')                     # write out the file 
    #graph.write_png('dbschema.png')                     # write out the file 
else:
    print "Not creating schema figure because 'sqlalchemy_schemadisplay' is not installed"

print 'all tests pass.'



#!/usr/bin/python
"""
Container class for gene sets generated from a labels file
"""

__author__ = "Adam Richards"

import os,re,sys,csv
import numpy as np
from htsint.blast import BlastMapper

class GeneSetCollection(object):
    """
    gene set class

    """

    def __init__(self, labelsFile, gene2go):
        """
        Constructor

        labelsFile: path to a csv with the following format
 
            k=300,sigma=0.5
            gene,label
            12345,0
            12346,1
            12347,2
            12349,0
            ...
        
        gene2go: a dictionary with the following format

            gene2go = {'100158436': ['GO:0008277', 'GO:0071108', 'GO:0016579',
                       '734137': ['GO:0006508'], 
                       '398019': ['GO:0007281']}

        """

        ## error check
        if not os.path.exists(labelsFile):
            raise Exception("cannot find labels file")

        self.genes,self.labels = self.read_labels(labelsFile)
        print("loaded file with %s genes and %s clusters"%(self.genes.size,np.unique(self.labels).size))
        
        if type(gene2go) != type({}):
            raise Exception("invalid gene2go dictionary")
        self.gene2go = gene2go

        self.allClusters = np.sort(np.unique(self.labels))
    
    def write(self,blastMap=None, sizeMin=4, sizeMax=100,outFile="genesets.gmt"):
        """
        outFile: specifies the output file path (*.gmt)
        also a *.csv file with gene transcript mapping will be created if a bmap is provided

        blastMap: BlastMap returned after loading summary file in BlastMapper 
        sizeMin: minimum size for a gene set
        sizeMax: maximum size for a gene set
        outFile: outfile path

        """

        ## prepare outfiles
        writer = csv.writer(open(outFile,'w'),delimiter="\t")

        if blastMap:
            outFileMap = re.sub("\.gmt",".csv",outFile)
            writerMap = csv.writer(open(outFileMap,'w'))
            writerMap.writerow(["gene_set","gene_id","mapped_transcripts"])

        ## prepare a blastmap
        if blastMap:
            bm = BlastMapper()
            transcript2gene,gene2transcript = bm.get_dicts(blastMap)

        ## save gene sets to file
        failedThreshold = 0
        realizedGenes = 0
        for _k in self.allClusters:

            numGenes = np.where(self.labels==_k)[0].size
            mappedGenes = set([])
            gsName = "gs-"+str(_k)
            clusterInds = np.where(self.labels==_k)[0]
            clusterGenes = self.genes[clusterInds]
            description = self.get_description(clusterGenes)

            ## map the genes
            if blastMap:
                for gene in clusterGenes:
                    geneTranscripts = set([])
                    if not gene2transcript.has_key(gene):
                        continue
            
                    for taxa,matchedTranscripts in gene2transcript[gene].iteritems():
                        mappedGenes.update(matchedTranscripts)
                        geneTranscripts.update(matchedTranscripts)
                    if blastMap:
                        writerMap.writerow([gsName,gene,";".join(list(geneTranscripts))])


            else:
                mappedGenes = clusterGenes

            mappedGenes = list(mappedGenes)
            realizedGenes += len(mappedGenes)

            if len(mappedGenes) >= sizeMin and len(mappedGenes) <= sizeMax: 
                writer.writerow([gsName,description] + mappedGenes)
            else:
                failedThreshold+=clusterGenes.size

        print("-----------------")
        print("sigma: %s"%self.sigma)
        print("k: %s"%self.k)
        print('Total clusters: %s '%self.allClusters.size)
        percentAccepted = float(self.genes.size-failedThreshold) / float(self.genes.size)
        print("Genes in clusters %s/%s (%s)"%(self.genes.size-failedThreshold,self.genes.size,round(percentAccepted,2)) + "%)")
    
    def get_description(self,geneList):
        """
        function the return description
        """

        terms = []
        for gene in geneList:
            terms.extend(self.gene2go[gene])

        terms = np.array(terms)
        uniqueTerms = np.sort(np.unique(terms))
        counts = []
        for t in uniqueTerms:
            counts.append(np.where(terms==t)[0].size)
        count = np.array(counts)
        rankedCounts = np.argsort(counts)[::-1]
        topThree = uniqueTerms[rankedCounts[:3]].tolist()

        return ';'.join(topThree)

    def read_labels(self, labelsFile):
        """
        read the labels file
        """

        ## read in the genes and labels
        fid = open(labelsFile,'r')
        reader = csv.reader(fid)
        header1 = reader.next()
        header2 = reader.next()
        self.k = header1[0].split("=")[1]
        self.sigma = float(header1[1].split("=")[1])
        
        genes,labels = [],[]
        for linja in reader:
            if len(linja) < 2:
                continue
            genes.append(linja[0])
            labels.append(linja[1])

        return np.array(genes),np.array(labels)
        
if __name__ == "__main__":
    print "Running..."



#!/usr/bin/env python

import os,sys,re
from distutils.sysconfig import get_python_lib
from shutil import copytree
from distutils.core import setup
#from numpy.distutils.core import setup

try:
    from py2exe.build_exe import py2exe
except:
    pass

sys.path.append("htsint")
from version import __version__

__author__ = "AJ Richards"

DESCRIPTION = "API and database for high-throughput sequencing analyses."
LONG_DESCRIPTION = """
A package that creates gene sets using unsupervised clustering. Gene sets are
generated based on Gene Ontology information from one or more species.

"""

#def get_files(dirPath):
#    included = ["\.py"]
#    allFiles = []
#    for fileName in os.listdir(dirPath):
#        include = False
#        for pat in included:
#            if re.search(pat,fileName):
#                include = True
#        filePath = os.path.join(dirPath,fileName)
#        if include == True and os.path.isfile(filePath):
#            allFiles.append(os.path.realpath(filePath))
#            
#    return allFiles

REQUIRES = ['numpy', 'matplotlib','networkx','sqlalchemy','biopython']
DISTNAME = 'htsint'
LICENSE = 'BSD 3-Clause'
AUTHOR = "Adam J Richards"
AUTHOR_EMAIL = "adamricha@gmail.com"
URL = 'https://github.com/ajrichards/hts-integrate'
CLASSIFIERS = [
    'Development Status :: 3 - Alpha',
    'Operating System :: OS Independent',
    'Intended Audience :: Science/Research',
    'Programming Language :: Python',
    'Topic :: Scientific/Engineering',
    'License :: OSI Approved :: BSD License'
]


FILES = {'htsint': [os.path.join('database','*.py'),
                    os.path.join('stats','*.py'),
                    os.path.join('blast','*.py'),
                    os.path.join('tools','*.py')]}

ISRELEASED = True
VERSION = __version__
FULLVERSION = VERSION
if not ISRELEASED:
    FULLVERSION += '.beta'


if __name__ == '__main__':
    setup(name=DISTNAME,
          author=AUTHOR,
          author_email=AUTHOR_EMAIL,
          description=DESCRIPTION,
          version=FULLVERSION,
          license=LICENSE,
          url=URL,
          packages=['htsint'],
          long_description=LONG_DESCRIPTION,
          classifiers=CLASSIFIERS,
          package_data=FILES,
          platforms='any')



#!/usr/bin/env python

import numpy as np
from scipy.spatial.distance import pdist, cdist, squareform

class SilValueGenerator():
    
    def __init__(self,mat,labels,dMetric='euclidean'):
        '''
        mat - is an n x d matrix of observations and features
        labels = is a vector of lenth n with a single assignment for each observation
        dMetric = is a given distance metric

        '''
        
        ## error checking
        matRows,matCols = np.shape(mat)
        if matRows != len(labels):
            print "INPUT ERROR: SilValueGenerator must have matching matrix and labels"

        self.mat = mat
        self.labels = np.array([l for l in labels])
        self.dMetric = dMetric

        ## get values 
        self.silValues = self._get_silhouette_values()


    def _get_silhouette_values(self):

        if self.dMetric != 'euclidean':
            print "ERROR: SilValueGenerator bad distance metric specified"
            return None

        ## get dims
        if len(self.mat.shape) == 1:
            d = 1
            n = self.mat.size
        else:
            n,d = self.mat.shape

        distInter = np.zeros((n,),)
        distIntra = np.zeros((n,),)
        silhouetteVals = None
        uniqueLabels = np.sort(np.unique(self.labels))
    
        ## get distances between cluster centers
        centroids = np.zeros((len(uniqueLabels),d))
        for ul in range(len(uniqueLabels)):
            ulab = uniqueLabels[ul]
            indicesK = np.where(self.labels==ulab)[0]
            centroids[ul,:] = self.mat[indicesK,:].mean(axis=0)

        centroidDMatrix = squareform(pdist(centroids,'euclidean'))
        for ul in range(len(uniqueLabels)):
            centroidDMatrix[ul,ul] = np.inf

        ## calculate distances 
        for ul in range(len(uniqueLabels)):
            ulab = uniqueLabels[ul]

            ## within values
            indicesK = np.where(self.labels==ulab)[0]
            elementsK = self.mat[indicesK,:]
            clustDists = pdist(elementsK,'euclidean')
            clustDists = squareform(clustDists)
            #means = np.zeros(len(indicesK,),)
            #for i in range(len(indicesK)):
            #    rowIndices = list(set(range(len(indicesK))).difference(set([i])))
            #    means[i] = clustDists[rowIndices,:].mean()

            means = clustDists.mean(axis=0)
            distIntra[indicesK] = means
            
            ## get closest cluster
            closeClust = uniqueLabels[np.where(centroidDMatrix[ul,:] == centroidDMatrix[ul,:].min())[0][0]]
            
            ## between values
            indicesClose = np.where(self.labels==closeClust)[0]
            elementsClose = self.mat[indicesClose,:]
            clustDists = cdist(elementsClose, elementsK,'euclidean')
            means = clustDists.mean(axis=0)
            distInter[indicesK] = means

        silhouetteVals = (distInter - distIntra) / np.maximum(distIntra, distInter)

        return silhouetteVals



#!/usr/bin/env python
import os,sys,time,re
import matplotlib.pyplot as plt
import numpy as np
import scipy.stats as stats

## create doc beginning
class LatexReportCreator():
    def __init__(self,docName,docType='report',docDir="report",verbose=False,extraPackages=[],generatedBy="Python"):
        self.docName = docName
        self.docType = docType
        self.docDir = docDir
        self.verbose = verbose
        self.generatedBy = generatedBy
        self.extraPackages = extraPackages
        self.fontSizeList = ["tiny","scriptsize","footnotesize","small","normalsize","large","Large","LARGE","huge","Huge"] 

        ## error check
        if type(self.extraPackages) != type([]):
            print "INPUT ERROR: extra packages must be in list form"

        if os.path.isdir(self.docDir) == False:
            if self.verbose == True:
                print "...making report directory"
            os.mkdir(self.docDir)
        
        self.fid = open(os.path.join(self.docDir,self.docName + ".tex"),'w')

    def write_beginning(self,title,author,shortTitle='report',shortAuthor='',toc=False):
        self.fid.write("\documentclass[letterpaper,10pt]{%s}\n"%self.docType)
        self.fid.write("\usepackage{pgf,graphics,color,hyperref,fullpage,natbib,algorithm,algorithmic,amsmath,ulem,bm,morefloats}\n")
        for package in self.extraPackages:
            self.fid.write("\usepackage{%s}\n"%package)
        self.fid.write("\definecolor{darkblue}{rgb}{0.0,0.0,0.50}\n")
        self.fid.write("\definecolor{darkgreen}{rgb}{0.0,0.35,0.0}\n")
        self.fid.write("\hypersetup{colorlinks=true, linkcolor=darkblue, citecolor=darkblue, urlcolor=darkblue}\n")
        self.fid.write("\hypersetup{pdfauthor={%s}, pdftitle={%s}}\n"%(shortAuthor,shortTitle))
        self.fid.write("\usepackage{titlesec}\\titleformat{\chapter}{\centering\\normalfont\Large\\bfseries}{\\thechapter}{1em}{}\n")

        ## beginning materials
        self.fid.write("\\begin{document}\n")
        self.fid.write("%%%%%\n")
        self.fid.write("\\begin{flushleft}\n")
        self.fid.write("\\textbf{%s}\\\ \n"%title)
        self.fid.write("Created by: %s\\\ \n"%author)
        self.fid.write("Created on: %s\n"%time.asctime())
        self.fid.write("\end{flushleft}\n")
        self.fid.write("\medskip\hrule height 1pt\n")
        self.fid.write("\\vspace{7pt}\n")

        if toc == True:
            self.fid.write("\\tableofcontents\n")

    ## convenience functions
    def begin(self,item):
        if item in self.fontSizeList:
            self.fid.write("\%s\n"%item)
        else:
            self.fid.write("\\begin{%s}\n"%item)

    def end(self,item):
        if item in self.fontSizeList:
            self.fid.write("\\normalsize\n")
        else:
            self.fid.write("\end{%s}\n"%item)

    def section(self,sectionName,sectionType="section",numbers=True,toc=False):
        '''
        toc - used to manually add a section to the table of contents -- must also call toc = True in 'write_beginning'

        '''

        if numbers == True:
            self.fid.write("\%s{%s}\n"%(sectionType,sectionName))
        else:
            self.fid.write("\%s*{%s}\n"%(sectionType,sectionName))

        if toc == True:
            self.fid.write("\\addcontentsline{toc}{%s}{%s}\n"%(sectionType,sectionName))

    ## main functions
    def write_paragraph(self,body):
        self.fid.write(body+"\n\n")

    def write_end(self):
        self.fid.write("\n\end{document}")
        self.fid.close()

    def compile(self):
        currentDir = os.getcwd()
        os.chdir(self.docDir)
        os.system("pdflatex %s"%self.docName+".tex")
        os.chdir(currentDir)

    def include_figure(self,figPath,caption=None,label=None,scale=0.65):
        if os.path.isfile(figPath) == False:
            print "ERROR: could not include figure does not exist", figPath

        self.fid.write("\\begin{figure}[!ht]\n")
        self.begin("center")
        self.fid.write("\includegraphics[scale = %s]{%s}\n"%(scale,figPath))
        if caption != None:
            self.fid.write("\caption{%s}\n"%caption)
        if label != None:
            self.fid.write("\label{fig:%s}\n"%label)
        self.end("center")
        self.end("figure")

    def include_table(self,rowDict,colHeader=None,customHeader=None,justifications=None,caption=None,label=None,fontSize='normalsize',ordered=False):
        '''
        rowDict - which has row header elements as keys and lists to fill out the cols as values
        colHeader - are the header elements for the columns
        '''
        
        ## if keys are only numeric
        numericKeys = False
        if not re.search('\D',str(rowDict.keys()[0])): 
            numericKeys = True

        ## error checking
        if type(rowDict) != type({}):
            print "ERROR: rowDict for include_table must be of type dict"
        if type(rowDict[rowDict.keys()[0]]) != type([]):
            print "ERROR: the values in rowDict for included_table must be of type list"

        numCols = len(rowDict[rowDict.keys()[0]])
        if justifications == None and numericKeys == False:
            justifications = '|l|'+'c'*numCols+"|"
        elif justifications == None and numericKeys == True:
            justifications = '|l|'+'c'*(numCols-1)+"|"
            
        self.fid.write('\\begin{table}[!h]\n')
        self.begin('center')
        self.begin(fontSize)
        self.fid.write("\\begin{tabular}{%s}\n"%justifications)
        self.fid.write("\\hline\n")

        if customHeader != None:
            self.fid.write(customHeader)
            self.fid.write("\\hline\n")
        
        if colHeader != None:
            header = "".join([i + "&" for i in colHeader])[:-1]+"\\\ \n"
            self.fid.write(header)
            self.fid.write("\\hline\n")

        if numericKeys == True:      
            sk = [int(k) for k in rowDict.keys()]
            sk.sort()
            for key in sk:
                row = rowDict[str(key)]
                if row[0] == "hline":
                    self.fid.write("\\hline\n")
                else:
                    row = "".join([i + "&" for i in row])[:-1]+"\\\ \n"
                    self.fid.write(row)
        elif type(ordered) == type([]):
            for rowHead in ordered:
                row = rowDict[rowHead]
                if rowHead == 'hline':
                    self.fid.write("\\hline\n")
                else:
                    row = rowHead + "&" + "".join([i + "&" for i in row])[:-1]+"\\\ \n"
                    self.fid.write(row)
        elif ordered == True:
            it = iter(sorted(rowDict.iteritems()))
            for rowHead, row in it:
                if row[0] == 'hline':
                    self.fid.write("\\hline\n")
                else:
                    row = rowHead + "&" + "".join([i + "&" for i in row])[:-1]+"\\\ \n"
                    self.fid.write(row)
        else:
            for rowHead, row in rowDict.iteritems():
                if row[0] == 'hline':
                    self.fid.write("\\hline\n")
                else:
                    row = rowHead + "&" + "".join([i + "&" for i in row])[:-1]+"\\\ \n"
                    self.fid.write(row)

        self.fid.write("\\hline\n")
        self.end('tabular')
        self.end(fontSize)
        self.end('center')
        if caption != None:
            self.fid.write("\caption{%s}\n"%caption)
        if label != None:
            self.fid.write("\label{tab:%s}\n"%label)
        self.end('table')

## run a small test 
if __name__ == '__main__':

    ## create a preambble
    report = LatexReportCreator('test',verbose=True)
    report.write_beginning("Test Report Title","Ender Wiggin",toc=True)
    
    ## create the body
    report.section("About",numbers=False,toc=True)
    
    report.write_paragraph("this is a paragraph blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah "+ 
                           "blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah")
    report.write_paragraph("this is a paragraph blah blah blah blah blah blah blah blah blah blah blah blah "+ 
                           "blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah")
   
    report.section("Results",numbers=False,toc=True)
    report.write_paragraph("this is a paragraph blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah "+
                           "blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah")
    report.write_paragraph("this is a paragraph blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah "+
                           "blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah")
   
    colHeader = ["I", "II", "III", "IV"] 
    rowDict = {"a":["1","2","3"],
               "b":["4","5","6"],
               "c":["7","8","9"]   
               }

    caption = "\\textbf{an example table}. this is a table caption"
    label = "example-table"
    report.include_table(rowDict,colHeader,caption=caption,label=label)

    ## create a test figure
    fig = plt.figure()
    ax = fig.add_subplot(111)
    n = 5000
    alpha,beta = 2.0,3.0
    samples = np.random.beta(alpha,beta,size=n)
    p1 =ax.hist(samples,normed=1,alpha=0.7)
    pdfX = np.linspace(0,1,300)
    pdfY = stats.beta.pdf(pdfX,alpha,beta)
    p2 = ax.plot(pdfX,pdfY,color='orange',linewidth=2.5)
    ax.set_xlim((0,1))

    ax.set_title(r"Beta distribution: $a$ = %s, $b$ = %s, $n$ = %s"%(alpha,beta,n))
    figPath = os.path.join(report.docDir,"examplefig.pdf")
    ax.legend([p2],["pdf"])
    plt.savefig(figPath)

    ## add in test figure
    caption = "\\textbf{The beta distribution}. this is a figure caption"
    label = "beta-distn"
    report.include_figure("examplefig.pdf",caption=caption,label=label,scale=0.5)

    ## add another paragraph and close report
    report.write_paragraph("This is an example of how to make a reference to Figure~\\ref{fig:beta-distn} blah blah blah blah blah "+
                           "blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah")

    report.write_end()
    report.compile()



#!/usr/bin/env python
"""
a class to handle results from BLAST searches

the parsed file that BlastMapper handles is a csv file with the following rows
    query,hit-identifier,hit-identifier-long,e-score,bit-score

    query is specific to the assembly
    HitId is a RefSeq

there is an example of this file in the unittest directory
It can be created with ParseBlast.py or ParallelParseBlast.py

"""

import os,sys,csv,re,getopt,time
import numpy as np
import matplotlib.pyplot as plt
from matplotlib import font_manager as fm
from sqlalchemy.sql import select
from htsint.database import db_connect,Taxon,Gene,Uniprot,uniprot_mapper

__author__ = "Adam Richards"

class BlastMapper(object):
    """
    Takes a parsed blast output and wraps it into a convenience class
    """

    def __init__(self):
        """
        Constructor
        """

        self.session,self.engine = db_connect()
        self.conn = self.engine.connect()
        self.hits = None

    def create_summarized(self,parsedFilePath,summaryFilePath=None,large=False):
        """
        htsint uses output 5 (XML) and then parses it into a simple csv file
        """
        
        ## error checking
        if not os.path.exists(parsedFilePath):
            raise Exception("cannot find parsed file")

        if summaryFilePath == None:
            summaryFilePath = re.sub("\.csv","",parsedFilePath)+ "_summary.csv"

        ## input/output
        fidin = open(parsedFilePath,'rU')
        reader = csv.reader(fidin)
        header = reader.next()

        ## prepare out file         
        fidout = open(summaryFilePath,'w')
        writer = csv.writer(fidout)
        writer.writerow(["queryId","hitId","hitNcbiId","hitSpecies","hitSpeciesNcbiId","e-value"])

        ## read through the blast file and extract a unique list of ids 
        uniprotEntries = set([])
        for linja in reader:
            hitIdLong = linja[2]
            _hitId  = hitIdLong.split(" ")[1].split("|")
            if _hitId[-1] == '':
                hitId = _hitId[-2]
            else:
                hitId = _hitId[-1]
                
            uniprotEntries.update([hitId])
                 
        uniprotEntries = list(uniprotEntries)
        fidin.close()

        print("batch querying %s UniProt entries in the database... this may take some time"%(len(uniprotEntries)))
        #results = self.conn.execute(mytable.__table__.select(mytable.value.in_(values))
        #vailable_values = set(row.value for row in results)

        timeStart = time.time()
        upEntry2Gene, upEntry2Taxa = {},{}
        if large == False:
            results = self.conn.execute(Uniprot.__table__.select(Uniprot.uniprot_entry.in_(uniprotEntries)))
            for row in results:
                upEntry2Gene[str(row.uniprot_entry)] = str(row.gene_id)
                upEntry2Taxa[str(row.uniprot_entry)] = str(row.taxa_id)
        else:
            ## using htsint's mapper
            uMapper = uniprot_mapper(self.session,uniprotIdList=uniprotEntries,gene=True,taxa=True)
            for key,item in uMapper.iteritems():
                upEntry2Gene[str(key)] = str(item['gene_id'])
                upEntry2Taxa[str(key)] = str(item['taxa_id'])
       
        ## query the taxa just to double check
        taxaList = list(set(upEntry2Taxa.values()))
        while 'None' in taxaList: taxaList.remove('None')
        s = select([Taxon.id,Taxon.ncbi_id,Taxon.name]).where(Taxon.id.in_([int(tid) for tid in taxaList]))
        _taxaQueries = self.conn.execute(s)
        taxaQueries = _taxaQueries.fetchall()
        taxaId2Ncbi = dict([(str(tquery['id']),str(tquery['ncbi_id'])) for tquery in taxaQueries])

        ## create a single dictionary of all gene information 
        gene2id = {}
        for taxaDbId in taxaList:
            s = select([Gene.id,Gene.ncbi_id],Gene.taxa_id==taxaDbId)
            _geneQueries = self.conn.execute(s)
            taxaDict = dict([(str(r['id']),str(r['ncbi_id'])) for r in _geneQueries.fetchall()])
            #print("there are  %s genes from %s (%s)"%(len(taxaDict.keys()),tquery['name'],tquery['ncbi_id']))
            gene2id.update(taxaDict)
        print("uniprot batch query: %s"%time.strftime('%H:%M:%S',time.gmtime(time.time()-timeStart)))

        ## read through the file again
        fidin = open(parsedFilePath,'rU')
        reader = csv.reader(fidin)
        header = reader.next()

        for linja in reader:
            query = linja[0]
            hitIdShort = linja[1]
            hitIdLong = linja[2]
            eScore = linja[3]
            bitScore = linja[4]
            queryId = query.split(" ")[0]            

            _hitId  = hitIdLong.split(" ")[1].split("|")
            if _hitId[-1] == '':
                hitId = _hitId[-2]
            else:
                hitId = _hitId[-1]
            
            hitNcbiId,hitSpeciesNcbiId = '-','-'
            hitSpecies = re.findall("OS=.+[A-Z]=",hitIdLong)[0][3:-4]
            if re.findall("[A-Z]=",hitSpecies):
                hitSpecies = hitSpecies[:re.search("[A-Z]=",hitSpecies).start()-2]

            ## map the uniprot id to gene
            if upEntry2Gene.has_key(hitId) and str(upEntry2Gene[hitId]) != 'None':
                if gene2id.has_key(upEntry2Gene[hitId]) and str(gene2id[upEntry2Gene[hitId]]) != 'None':
                    hitNcbiId = gene2id[upEntry2Gene[hitId]]
       
            ## get taxa id associated with uniprot id
            hitSpeciesNcbiId = '-'
            if upEntry2Taxa.has_key(hitId) and str(upEntry2Taxa[hitId]) != 'None':
                if taxaId2Ncbi.has_key(upEntry2Taxa[hitId]) and str(taxaId2Ncbi.has_key(upEntry2Taxa[hitId])) != 'None':
                    hitSpeciesNcbiId = taxaId2Ncbi[upEntry2Taxa[hitId]]
                else:
                    print 'Were in! but taxaId2Ncbi does not have', upEntry2Taxa[hitId]
            else:
                print 'upEntry2Taxa does not have', hitId
            writer.writerow([queryId,hitId,hitNcbiId,hitSpecies,hitSpeciesNcbiId,eScore])

        fidin.close()
        fidout.close()

    def load_summary(self,filePath,taxaList=None,trinityGene=False,evalue=0.0001,best=True):
        """
        Because BLAST searches can result in large results lists there are several options
        
        filePath - path to summaryFile
        taxaList - if specified ignore any taxa not found in list otherwise take all taxa
        
        At this point the function returns only the best match for a each taxa

        A summary results file has the following header:
        queryId,hitId,hitNcbiId,hitSpecies,hitSpeciesNcbiId,e-value

        trinityGene (default = False) - function assumes that each queryId is associated with a unique identifier
                                           if set to True using Trinity nameing scheme results become 'gene' centric
        
        best (default = True) - if specified as True then only the query results with the lowest e-value is returned
                                otherwise all results are returned

        """

        ## error checking
        if not os.path.exists(filePath):
            raise Exception("cannot find summary file")

        ## if taxa filter provided
        if taxaList != None:
            s = select([Taxon.id,Taxon.ncbi_id,Taxon.name]).where(Taxon.ncbi_id.in_(taxaList))
            _taxaQueries = self.conn.execute(s)
            taxaQueries = _taxaQueries.fetchall()
            selectedTaxa = [str(tquery['id']) for tquery in taxaQueries]
            taxa2name = dict([(str(tquery['id']),str(tquery['ncbi_id'])) for tquery in taxaQueries])

        ## read the results
        results = {}
        fid = open(filePath,'rU')
        reader = csv.reader(fid)
        header = reader.next()

        uniqueQueries = set([])
        totalQueries = 0
        totalHits = 0
        evalueFilter = 0
        taxaFilter = 0
        for linja in reader:
            if len(linja) == 6:
                queryId = linja[0]
                hitId = linja[1]
                hitNcbiId = linja[2]
                hitSpecies = linja[3]
                hitSpeciesNcbiId = linja[4]
                _evalue = float(linja[5])
            else:
                raise Exception("Invalid number of columns in line in summary blast file \n%s"%linja)

            if trinityGene == True:
                queryId = re.sub("_i\d+","",queryId)

            totalHits += 1

            # filtering
            if '-' in linja:
                continue
            if _evalue > evalue:
                evalueFilter += 1
                continue

            if taxaList and hitSpeciesNcbiId not in taxaList:
                taxaFilter += 1
                continue

            ## use the best evalue 
            if not results.has_key(queryId):
                if best:
                    results[queryId] = (hitId,hitNcbiId,hitSpecies,hitSpeciesNcbiId,_evalue)
                else:
                    results[queryId] = [(hitId,hitNcbiId,hitSpecies,hitSpeciesNcbiId,_evalue)]

            else:
                ## if not in append mode and we have smaller evalue
                if best and _evalue < results[queryId][4]:
                    results[queryId] = (hitId,hitNcbiId,hitSpecies,hitSpeciesNcbiId,_evalue)
                ## if append mode and we have smaller evalue
                elif (hitId,hitNcbiId,hitSpecies,hitSpeciesNcbiId,_evalue) in results[queryId]:
                    continue
                elif not best and _evalue < results[queryId][0][4]:
                    results[queryId] = [(hitId,hitNcbiId,hitSpecies,hitSpeciesNcbiId,_evalue)] + results[queryId]
                elif not best and _evalue >= results[queryId][0][4]:
                    results[queryId].append((hitId,hitNcbiId,hitSpecies,hitSpeciesNcbiId,_evalue))

        print("queries filtered due to evalue > %s: %s"%(evalue,evalueFilter))
        print("queries filtered due to taxa: %s"%(taxaFilter))
        print("total hits: %s"%totalHits)
        self.hits = totalHits
        
        return results


    def make_taxa_pie_chart_and_table(self,bmap,removeStrain=False,threshold=2,figName=None,csvName=None,
                                      ax=None,shadow=True,startangle=90,explode=True,fontSize=11):
        """
        summarize the taxa using a pie chart and reStructuredText table
        
        all species that make up less than threshold percent will be grouped as 'other'

        """

        taxaHits = {}
        for transcriptId, items in bmap.iteritems():
            hitId,hitNcbiId,hitSpecies,hitSpeciesNcbiId,evalue = items
    
            if removeStrain:
                hitSpecies = re.sub("\s+$","",re.sub("\(.+\)","",hitSpecies))

            if not taxaHits.has_key(hitSpecies):
                taxaHits[hitSpecies] = 0
            taxaHits[hitSpecies] += 1

        taxaNames = np.array(taxaHits.keys())
        taxaCounts = np.array(taxaHits.values()).astype(float)
        percents = (taxaCounts / taxaCounts.sum()) * 100.0

        ## reorder as ranked
        rankedInds = np.argsort(taxaCounts)[::-1]
        taxaNames = taxaNames[rankedInds]
        taxaCounts = taxaCounts[rankedInds]
        percents = percents[rankedInds]

        ## create pie plot
        includedInds = np.where(percents >= threshold)[0]
        otherPercent = percents[np.where(percents < threshold)[0]].sum()
        
        if otherPercent > 0.0:
            labels = taxaNames[includedInds].tolist() + ['other']
            sizes = percents[includedInds].tolist() + [otherPercent]
        else:
            labels = taxaNames[includedInds].tolist()
            sizes = percents[includedInds].tolist()
       
        ## explode the biggest chunk
        if explode:
            explodeInd = sizes.index(max(sizes))
            explode = np.zeros(len(sizes))
            explode[explodeInd] = 0.1
        else:
            explode = None

        if ax == None:
            fig = plt.figure(figsize=(9,7))
            ax = fig.add_subplot(111)
     
        colors = ['yellowgreen', 'gold', 'lightskyblue', 'lightcoral','lightgray',
                  'khaki','honeydew','tomato','cornflowerblue','darkseagreen','darkorchid'] * 100

        ## shorten long names
        longNames = {"Schizosaccharomyces pombe":"S. pombe"}
        for ln1,ln2 in longNames.iteritems():
            if ln1 in labels:
                lnind = labels.index(ln1)
                labels[lnind] = ln2
            
        patches, texts, autotexts =ax.pie(sizes, labels=labels,explode=explode,colors=colors,
                                          autopct='%1.1f%%', shadow=shadow, startangle=startangle)        
        ax.axis('equal')
        
        ## adjust font sizes
        proptease = fm.FontProperties()
        proptease.set_size(fontSize)
        plt.setp(autotexts, fontproperties=proptease)
        plt.setp(texts, fontproperties=proptease)

        if figName:
            plt.savefig(figName)

        print 'total species:', len(taxaHits.keys())
        print 'check', np.array(sizes).sum()

        ## save results as csv file
        if not csvName:
            return

        fid = open(csvName,'w')
        writer = csv.writer(fid)
        writer.writerow(["SpeciesName","TotalHits","Percentage"])
        for i in range(taxaNames.size):
            writer.writerow([taxaNames[i],taxaCounts[i],percents[i]])

        fid.close()

    def get_dicts(self,bmap):
        """
        returns a gene to transcript and transcript to gene dictionary
        """
         
        transcript2gene = {}
        gene2transcript = {}
        for key,_item in bmap.iteritems():
            for item in _item:
                if item[1] != '-':
                    transcript2gene[key] = item[1]
                    if not gene2transcript.has_key(item[1]):
                        gene2transcript[item[1]] = {}
                    if not gene2transcript[item[1]].has_key(item[3]):
                        gene2transcript[item[1]][item[3]] = set([])
                    gene2transcript[item[1]][item[3]].update([key])
        return transcript2gene,gene2transcript  


if __name__ == "__main__":
    print "Running..."



#!/usr/bin/python

import time,csv,re,sys
import numpy as np
from DatabaseTools import populate_gene_table
from htsint.database import  get_file_sizes, db_connect

session,engine = db_connect()
timeStart = time.time()
idmapCount,geneInfoCount = get_file_sizes()

#timeStr,addedStr = populate_gene_table(geneInfoCount,session,engine)
#push_out(timeStr)
#push_out(addedStr)


sys.exit()

uniprotIdMap = uniprot_mapper(session)

uniprotQuery = session.query(Uniprot).all()#filter_by(uniprot_id='P07663').first()
#self.assertEqual(uniprotQuery.uniprot_entry,"PER_DROME")
#geneQuery = self.session.query(Gene).filter_by(id=uniprotQuery.gene_id).first(
#elf.assertEqual(geneQuery.ncbi_id,'31251')

for uq in uniprotQuery:
    print 'id',uq.id
    print 'ac',uq.uniprot_ac


print len(uniprotQuery)

#print uniprotIdMap.keys()
sys.exit()


annotationFile = get_annotation_file()
annotationFid = open(annotationFile,'rU')
#gene2goFile = get_gene2go_file()
#gene2goFid = open(gene2goFile,'rU')
annotationCount = 0


print("...loading mappers")

noTaxon = 0
total = 0

print("...getting annotations from gene_association (uniprot)")
for record in annotationFid:
    record = record[:-1].split("\t")

    ## check that it is a uniprot entry 
    if record[0][0] == "!":
        continue
    if record[0] != 'UniProtKB':
        continue

    uniprotId = record[1]
    dbObjectSymbol = record[2]
    goId = record[4]
    pubmedRefs = record[5]
    evidenceCode = record[6]
    aspect = record[8]
    goTermName = record[11]
    taxon = re.sub("taxon:","",record[12])
    date = record[13]
    assignedBy = record[14]

    if re.search("\|",taxon):
        continue

    if taxon == None:
        noTaxon += 1

    total += 1

    print 'goid', goId
    print 'evidence code', evidenceCode
    print 'pubmedrefs', pubmedRefs
    print 'uniprotid', uniprotId
    print 'taxon', taxon

    if total > 25:
        sys.exit()


print 'annotation fid'
print 'total', total
print 'no taxon', noTaxon


#termIdMap = goterm_mapper(session)
#taxaIdMap = taxa_mapper(session)
#uniprotIdMap = uniprot_mapper(session)
#print("...populating rows")





"""
print("...getting gene info")
#geneIdMap = gene_mapper(session)
timeStart = time.time()
toAdd = []
totalRecords = 0
idmappingFile = "/usr/local/share/htsint/idmapping.dat.db"
idmappingFid = open(idmappingFile,'rb')
reader = csv.reader(idmappingFid,delimiter="\t")

print("...populating rows")
bad = 0
current = None



uniprotKbEntry,ncbiId,refseq,ncbiTaxaId = None,None,None,None

noTaxa = 0
noNcbi = 0
total = 0
noTaxaYesNcbi = 0

for record in reader:

    if len(record) != 3:
        continue

    uniprotKbAc = record[0]

    if current == None:
        current = uniprotKbAc

    if record[1] == 'NCBI_TaxID':
        ncbiTaxaId = record[2]
    if record[1] == 'GeneID':
        ncbiId = record[2]
    if record[1] == 'UniProtKB-ID':
        uniprotKbEntry = record[2]
    if record[1] == 'RefSeq':
        refseq = record[2]

    ## check to see if entry is finished
    if current != uniprotKbAc:
        current = uniprotKbAc
        total += 1

        if ncbiTaxaId == None:
            noTaxa += 1
        if ncbiId == None:
            noNcbi += 1
        if ncbiId and ncbiTaxaId == None:
            noTaxaYesNcbi += 1





        #if None in [uniprotKbEntry,ncbiId,refseq,ncbiTaxaId]:
        #    print "\nuniprotid:%s\nncbiid%s\nrefseq:%s\ntaxaid:%s"%(uniprotKbEntry,ncbiId,refseq,ncbiTaxaId)
        #else:
        #    print "...", uniprotKbEntry
        #sys.exit()

        uniprotKbEntry,ncbiId,refseq,ncbiTaxaId = None,None,None,None


print "......"
print "total%s"%total
print "noTaxa%s"%noTaxa
print "noNcbi%s"%noNcbi
print "noTaxaYesNcbi%s"%noTaxaYesNcbi
"""



"""
blast related classes and functions

"""

from RestLib import print_rest_table_contents
from LatexLib import LatexReportCreator
from ExpressionLib import read_RSEM_counts_files,read_matrix,read_de_results
from Heatmap import Heatmap



#!/usr/bin/env python
"""
produce a clustering heatmap

http://docs.scipy.org/doc/scipy-0.14.0/reference/cluster.hierarchy.html

"""

import sys
import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
from scipy.spatial.distance import pdist, squareform
from scipy.cluster.hierarchy import linkage, dendrogram


__author__ = "Adam Richards"

def blue_black_yellow():
    cdict = {'red':   ((0.0, 0.0, 0.0),\
                       (0.5, 0.0, 0.1),\
                       (1.0, 1.0, 1.0)),\

             'green': ((0.0, 0.0, 0.0),\
                       (0.5, 0.1, 0.0),\
                       (1.0, 1.0, 1.0)),\

             'blue':  ((0.0, 0.0, 1.0),
                       (0.5, 0.1, 0.0),
                       (1.0, 0.0, 0.0))
            }
    my_cmap = mpl.colors.LinearSegmentedColormap('my_colormap',cdict,256)
    return my_cmap

def red_black_green():
    cdict = {'red': ((0.0, 0.0, 0.0),\
                     (0.5, 0.0, 0.1),\
                     (1.0, 1.0, 1.0)),\

             'green': ((0.0, 0.0, 1.0),\
                       (0.5, 0.1, 0.0),\
                       (1.0, 0.0, 0.0)),

             'blue': ((0.0, 0.0, 0.0),\
                      (1.0, 0.0, 0.0))
            }

    my_cmap = mpl.colors.LinearSegmentedColormap('my_colormap',cdict,256)
    return my_cmap

def red_black_blue():
    cdict = {'red':   ((0.0, 0.0, 0.0),
                       (0.5, 0.0, 0.1),
                       (1.0, 1.0, 1.0)),

             'green': ((0.0, 0.0, 0.0),
                       (1.0, 0.0, 0.0)),

             'blue':  ((0.0, 0.0, 1.0),
                       (0.5, 0.1, 0.0),
                       (1.0, 0.0, 0.0))
            }

    my_cmap = mpl.colors.LinearSegmentedColormap('my_colormap',cdict,256)
    return my_cmap

class Heatmap(object):
    """
    produce a clustering heatmap

    linkage metrics = 'average', 'single', 'centroid', 'complete'

    """

    def __init__(self,width=6,height=7,rowLabels=[],colLabels=[],title=None,dpi=300,
                 hpad=0.1,vpad=0.05,fontSize=10,fontName='sans-serif'):
        """ 
        Constructor
        """

        self.indx = {}
        self.z = {}
        self.fontSize = fontSize
        self.fontName = fontName
        

        if type(rowLabels) == type([]):
            self.rowLabels = np.array(rowLabels)
        else:
            self.rowLabels = rowLabels
        if type(colLabels)== type([]):
            self.colLabels = np.array(colLabels)
        else:
            self.colLabels = colLabels

        ## initialize the axis (l,b,w,h)
        self.plt = plt
        self.fig = self.plt.figure(figsize=(width,height),dpi=dpi)

        ## set relative size of subplots (sum to 1.0)
        base1 = 0.15
        base2 = 0.85

        ## top dendogram
        self.ax1 = self.fig.add_axes([base1,vpad+base2,base2-hpad,base1-vpad])
        self.ax1.set_yticks([])
        self.ax1.set_xticks([])
        self.ax1.set_frame_on(False)

        ## side dendogram
        self.ax2 = self.fig.add_axes([0,vpad,base1,base2])
        self.ax2.set_yticks([])
        self.ax2.set_xticks([])
        self.ax2.set_frame_on(False)

        ## heatmap
        self.ax3 = self.fig.add_axes([base1,vpad,base2-hpad,base2])
        self.ax3.set_yticks([])
        self.ax3.set_xticks([])
        self.ax3.set_frame_on(False)

        ## colorbar
        hsize = base1           # same as side dendogram width
        vsize = base1-vpad      # same as top dendogram height
        left = 0.1 * hsize
        bottom = vpad+base2 + (0.4 * vsize)
        width = 0.80 * hsize
        height = 0.5 * vsize
        self.ax4 = self.fig.add_axes([left,bottom,width,height],frame_on=False)
        self.ax4.set_yticks([])
        self.ax4.set_xticks([])
        self.ax4.set_frame_on(False)

        self.title = None

    def cluster(self,mat,dim,labels=None):
        """
        use hierarchical clustering to group the data
        dim = 0 are the rows
        dim = 1 are the columns
        """
        
        if dim == 0:
            print("clustering the rows...")
            x = mat
            orientation='right'
            self.plt.sca(self.ax2)
            ax = self.ax2
        if dim == 1:
            print("clustering the columns...")
            x = mat.transpose()
            orientation='top'
            self.plt.sca(self.ax1)
            ax = self.ax1

        distMatrix = pdist(x)
        distMatrix = squareform(distMatrix)
        linkageMatrix = linkage(distMatrix,method='complete')

        z = dendrogram(linkageMatrix,orientation=orientation,\
                       no_labels=True,color_threshold=1.0,\
                       link_color_func=lambda k: 'k')

        indx = z['leaves']
        self.indx[str(dim)] = indx
        self.z[str(dim)] = z

    def draw_heatmap(self,mat,cmap='uy',clabels=True,rlabels=False,rowFont=None,colFont=None):
        """
        draw the heatmap portion of the plot
        cmap can be a custom instance of a cmap
        or 'yu' for yellow-black-blue'
        or 'rg' for red-black-green
        """
        
        if cmap == 'rg':
            cmap = red_black_green()
        elif cmap == 'uy':
            cmap = blue_black_yellow()
        else:
            cmap = cmap

        if rowFont == None:
            rowFont = self.fontSize-2
        if colFont == None:
            colFont = self.fontSize

        self.plt.sca(self.ax3)
        ax = self.ax3

        n,m = mat.shape
        
        if not self.indx.has_key('0') or not self.indx.has_key('1'):
            raise Exception("cluster before plotting heatmap")

        ## setup event handler
        def mat_picker(x, mouseevent):
            """ 
            find the row and column
            """

            if mouseevent.xdata == None or mouseevent.ydata == None:
                return False, dict()

            colCoord = np.floor(mouseevent.xdata+.5)
            rowCoord = np.floor(mouseevent.ydata+.5)
            
            
            if len(self.colLabels) > 0:
                col = self.colLabels[self.indx['1']][colCoord]
            else:
               col = np.arange(m)[self.indx['1']][colCoord]

            if len(self.rowLabels) > 0:
                row = self.rowLabels[self.indx['0']][rowCoord]
            else:
                row = np.arange(n)[self.indx['0']][rowCoord]

            print("row: %s (%s), col: %s (%s)"%(col,int(colCoord),row,int(rowCoord)))
            return False, dict()

        ## reorder matrix
        matReordered = mat[self.indx['0'],:]
        matReordered = matReordered[:,self.indx['1']]
        hmap = ax.imshow(matReordered, interpolation='nearest',aspect='auto',cmap=cmap,picker=mat_picker,origin='lower')

        ## handle axes
        if clabels and len(self.colLabels) > 0:
            ax.set_xticks(range(m))
            ax.set_xticklabels(self.colLabels[self.indx['1']],fontsize=colFont,fontname=self.fontName,rotation='vertical')
        
        if rlabels and len(self.rowLabels) > 0:
                
            if n > 200:
                print ("WARNING: too many rows to visualize row labels")
                rowFont = None

            if rowFont:
                ax.yaxis.set_ticks_position('right')
                ax.set_yticks(range(n))
                ax.set_yticklabels(self.rowLabels[self.indx['0']],fontsize=rowFont,fontname=self.fontName)

        
        ## colorbar
        self.plt.sca(self.ax4)
        ax = self.ax4

        val = np.ceil(np.abs(mat).max())
        norm = mpl.colors.Normalize(vmin=-1*val, vmax=val)
        cb1 = mpl.colorbar.ColorbarBase(ax,cmap=cmap,
                                        ticks=[int(round(i)) for i in np.linspace(-1*val,val,5)],
                                        norm=norm,
                                        orientation='horizontal')

        for t in ax.get_xticklabels():
            t.set_fontsize(self.fontSize)
            t.set_fontname(self.fontName)
        for t in ax.get_yticklabels():
            t.set_fontsize(self.fontSize)
            t.set_fontname(self.fontName)

        #print "min: %s,max: %s,mean: %s"%(round(mat.min(),2),round(mat.max(),2),round(mat.mean(),2))

    def save(self,fileName,dpi=300):
        """
        save the current plot to file
        """

        self.plt.savefig(fileName,dpi=dpi)

    def show(self):
        """
        display the current plot
        """

        self.plt.show()

if __name__ == "__main__":
    print "Running..."


    n = 10
    m = 6

    mat = np.vstack((np.random.normal(0,1,(n,m)),np.random.normal(3,1,(n,m))))
    mat[:,3:] = mat[:,3:] -2.0

    hm = Heatmap(colLabels=np.array(["A","B","C","D","E","F"]),\
                 rowLabels= np.array(["r"+str(i) for i in range(n*2)]))
    hm.cluster(mat,0)
    hm.cluster(mat,1)
    hm.draw_heatmap(mat,cmap='uy',clabels = True, rlabels=True)

    ## error checking
    hm.show()
